@@ -3,7 +3,7 @@
   hash.c -
 
   $Author: matz $
-  $Date: 2004/10/02 03:50:49 $
+  $Date: 2005/07/19 08:25:37 $
   created at: Mon Nov 22 18:51:18 JST 1993
 
   Copyright (C) 1993-2003 Yukihiro Matsumoto
@@ -121,17 +121,14 @@ struct foreach_safe_arg {
 };
 
 static int
-foreach_safe_i(key, value, arg, err)
+foreach_safe_i(key, value, arg)
     st_data_t key, value;
     struct foreach_safe_arg *arg;
 {
     int status;
 
-    if (err) {
-	rb_raise(rb_eRuntimeError, "hash modified during iteration");
-    }
     if (key == Qundef) return ST_CONTINUE;
-    status = (*arg->func)(key, value, arg->arg, err);
+    status = (*arg->func)(key, value, arg->arg);
     if (status == ST_CONTINUE) {
 	return ST_CHECK;
     }
@@ -149,7 +146,9 @@ st_foreach_safe(table, func, a)
     arg.tbl = table;
     arg.func = func;
     arg.arg = a;
-    st_foreach(table, foreach_safe_i, (st_data_t)&arg);
+    if (st_foreach(table, foreach_safe_i, (st_data_t)&arg)) {
+	rb_raise(rb_eRuntimeError, "hash modified during iteration");
+    }
 }
 
 struct hash_foreach_arg {
@@ -159,17 +158,13 @@ struct hash_foreach_arg {
 };
 
 static int
-hash_foreach_iter(key, value, arg, err)
+hash_foreach_iter(key, value, arg)
     VALUE key, value;
     struct hash_foreach_arg *arg;
-    int err;
 {
     int status;
     st_table *tbl;
 
-    if (err) {
- 	rb_raise(rb_eRuntimeError, "hash modified during iteration");
-    }
     tbl = RHASH(arg->hash)->tbl;    
     if (key == Qundef) return ST_CONTINUE;
     status = (*arg->func)(key, value, arg->arg);
@@ -207,7 +202,9 @@ static VALUE
 hash_foreach_call(arg)
     struct hash_foreach_arg *arg;
 {
-    st_foreach(RHASH(arg->hash)->tbl, hash_foreach_iter, (st_data_t)arg);
+    if (st_foreach(RHASH(arg->hash)->tbl, hash_foreach_iter, (st_data_t)arg)) {
+ 	rb_raise(rb_eRuntimeError, "hash modified during iteration");
+    }
     return Qnil;
 }
 
@@ -883,8 +880,6 @@ static VALUE
 rb_hash_clear(hash)
     VALUE hash;
 {
-    void *tmp;
-
     rb_hash_modify(hash);
     if (RHASH(hash)->tbl->num_entries > 0) {
 	rb_hash_foreach(hash, clear_i, 0);
@@ -1503,22 +1498,6 @@ rb_hash_equal(hash1, hash2)
     return hash_equal(hash1, hash2, Qfalse);
 }
 
-/*
- *  call-seq:
- *     hsh.eql?(other_hash)    => true or false
- *  
- *  Returns true if two hashes are equal, i.e they have same key-value set,
- *  and same default values.
- *     
- */
-
-static VALUE
-rb_hash_eql(hash1, hash2)
-    VALUE hash1, hash2;
-{
-    return hash_equal(hash1, hash2, Qtrue);
-}
-
 static int
 rb_hash_invert_i(key, value, hash)
     VALUE key, value;
@@ -1576,8 +1555,10 @@ rb_hash_update_block_i(key, value, hash)
 
 /*
  *  call-seq:
- *     hsh.merge!(other_hash)    => hsh
- *     hsh.update(other_hash)    => hsh
+ *     hsh.merge!(other_hash)                                 => hsh
+ *     hsh.update(other_hash)                                 => hsh
+ *     hsh.merge!(other_hash){|key, oldval, newval| block}    => hsh
+ *     hsh.update(other_hash){|key, oldval, newval| block}    => hsh
  *  
  *  Adds the contents of <i>other_hash</i> to <i>hsh</i>, overwriting
  *  entries with duplicate keys with those from <i>other_hash</i>.
@@ -1603,7 +1584,8 @@ rb_hash_update(hash1, hash2)
 
 /*
  *  call-seq:
- *     hsh.merge(other_hash) -> a_hash
+ *     hsh.merge(other_hash)                              -> a_hash
+ *     hsh.merge(other_hash){|key, oldval, newval| block} -> a_hash
  *  
  *  Returns a new hash containing the contents of <i>other_hash</i> and
  *  the contents of <i>hsh</i>, overwriting entries in <i>hsh</i> with
@@ -1838,7 +1820,7 @@ ruby_setenv(name, value)
     else
 	unsetenv(name);
 #else  /* WIN32 */
-
+    size_t len;
     int i=envix(name);		        /* where does it go? */
 
     if (environ == origenviron) {	/* need we copy environment? */
@@ -1853,30 +1835,28 @@ ruby_setenv(name, value)
 	tmpenv[max] = 0;
 	environ = tmpenv;		/* tell exec where it is now */
     }
-    if (!value) {
-	if (environ != origenviron) {
-	    char **envp = origenviron;
-	    while (*envp && *envp != environ[i]) envp++;
-	    if (!*envp)
-		free(environ[i]);
+    if (environ[i]) {
+	char **envp = origenviron;
+	while (*envp && *envp != environ[i]) envp++;
+	if (!*envp)
+	    free(environ[i]);
+	if (!value) {
+	    while (environ[i]) {
+		environ[i] = environ[i+1];
+		i++;
+	    }
+	    return;
 	}
-	while (environ[i]) {
-	    environ[i] = environ[i+1];
-	    i++;
-	}
-	return;
     }
-    if (!environ[i]) {			/* does not exist yet */
+    else {			/* does not exist yet */
+	if (!value) return;
 	REALLOC_N(environ, char*, i+2);	/* just expand it a bit */
 	environ[i+1] = 0;	/* make sure it's null terminated */
     }
-    else {
-	if (environ[i] != origenviron[i])
-	    free(environ[i]);
-    }
-    environ[i] = ALLOC_N(char, strlen(name) + strlen(value) + 2);
+    len = strlen(name) + strlen(value) + 2;
+    environ[i] = ALLOC_N(char, len);
 #ifndef MSDOS
-    sprintf(environ[i],"%s=%s",name,value); /* all that work just for this */
+    snprintf(environ[i],len,"%s=%s",name,value); /* all that work just for this */
 #else
     /* MS-DOS requires environment variable names to be in uppercase */
     /* [Tom Dinger, 27 August 1990: Well, it doesn't _require_ it, but
@@ -1904,7 +1884,7 @@ env_aset(obj, nm, val)
     char *name, *value;
 
     if (rb_safe_level() >= 4) {
-	rb_raise(rb_eSecurityError, "cannot change environment variable");
+	rb_raise(rb_eSecurityError, "can't change environment variable");
     }
 
     if (NIL_P(val)) {
@@ -2450,7 +2430,6 @@ Init_Hash()
     rb_define_method(rb_cHash,"inspect", rb_hash_inspect, 0);
 
     rb_define_method(rb_cHash,"==", rb_hash_equal, 1);
-    rb_define_method(rb_cHash,"eql?", rb_hash_eql, 1);
     rb_define_method(rb_cHash,"[]", rb_hash_aref, 1);
     rb_define_method(rb_cHash,"fetch", rb_hash_fetch, -1);
     rb_define_method(rb_cHash,"[]=", rb_hash_aset, 2);
