@@ -47,12 +47,6 @@
 #define bool int
 #endif
 
-#ifdef _M_IX86
-# define WIN95 1
-#else
-# undef  WIN95
-#endif
-
 #if defined __BORLANDC__ || defined _WIN32_WCE
 #  define _filbuf _fgetc
 #  define _flsbuf _fputc
@@ -209,10 +203,10 @@ map_errno(DWORD winerr)
 static char *NTLoginName;
 
 #ifdef WIN95
-DWORD Win32System = (DWORD)-1;
+static DWORD Win32System = (DWORD)-1;
 
-static DWORD
-IdOS(void)
+DWORD
+rb_w32_osid(void)
 {
     static OSVERSIONINFO osver;
 
@@ -224,21 +218,11 @@ IdOS(void)
     }
     return (Win32System);
 }
-
-static int 
-IsWin95(void) {
-    return (IdOS() == VER_PLATFORM_WIN32_WINDOWS);
-}
-
-static int
-IsWinNT(void) {
-    return (IdOS() == VER_PLATFORM_WIN32_NT);
-}
-#else
-# define IsWinNT() TRUE
-# define IsWin95() FALSE
 #endif
 
+#define IsWinNT() rb_w32_iswinnt()
+#define IsWin95() rb_w32_iswin95()
+
 /* main thread constants */
 static struct {
     HANDLE handle;
@@ -1728,9 +1712,9 @@ rb_w32_strerror(int e)
 	buffer[sizeof(buffer) - 1] = 0;
     }
 
-    for (p = buffer + strlen(buffer) - 1; buffer <= p; p--) {
-	if (*p != '\r' && *p != '\n') break;
-	*p = 0;
+    p = buffer;
+    while ((p = strpbrk(p, "\r\n")) != NULL) {
+	memmove(p, p + 1, strlen(p));
     }
     return buffer;
 }
@@ -1797,6 +1781,7 @@ int
   ioctl(int i, unsigned int u, long data)
 #endif
 {
+    errno = EINVAL;
     return -1;
 }
 
@@ -2824,6 +2809,111 @@ isUNCRoot(const char *path)
     return 0;
 }
 
+static time_t
+filetime_to_unixtime(const FILETIME *ft)
+{
+    FILETIME loc;
+    SYSTEMTIME st;
+    struct tm tm;
+    time_t t;
+
+    if (!FileTimeToLocalFileTime(ft, &loc)) {
+	return 0;
+    }
+    if (!FileTimeToSystemTime(&loc, &st)) {
+	return 0;
+    }
+    memset(&tm, 0, sizeof(tm));
+    tm.tm_year = st.wYear - 1900;
+    tm.tm_mon = st.wMonth - 1;
+    tm.tm_mday = st.wDay;
+    tm.tm_hour = st.wHour;
+    tm.tm_min = st.wMinute;
+    tm.tm_sec = st.wSecond;
+    t = mktime(&tm);
+    return t == -1 ? 0 : t;
+}
+
+static unsigned
+fileattr_to_unixmode(DWORD attr, const char *path)
+{
+    unsigned mode = 0;
+
+    if (attr & FILE_ATTRIBUTE_READONLY) {
+	mode |= S_IREAD;
+    }
+    else {
+	mode |= S_IREAD | S_IWRITE | S_IWUSR;
+    }
+
+    if (attr & FILE_ATTRIBUTE_DIRECTORY) {
+	mode |= S_IFDIR | S_IEXEC;
+    }
+    else {
+	mode |= S_IFREG;
+    }
+
+    if (path && (mode & S_IFREG)) {
+	const char *end = path + strlen(path);
+	while (path < end) {
+	    end = CharPrev(path, end);
+	    if (*end == '.') {
+		if ((strcmpi(end, ".bat") == 0) ||
+		    (strcmpi(end, ".cmd") == 0) ||
+		    (strcmpi(end, ".com") == 0) ||
+		    (strcmpi(end, ".exe") == 0)) {
+		    mode |= S_IEXEC;
+		}
+		break;
+	    }
+	}
+    }
+
+    mode |= (mode & 0700) >> 3;
+    mode |= (mode & 0700) >> 6;
+
+    return mode;
+}
+
+static int
+winnt_stat(const char *path, struct stat *st)
+{
+    HANDLE h;
+    WIN32_FIND_DATA wfd;
+
+    memset(st, 0, sizeof(struct stat));
+    st->st_nlink = 1;
+
+    if (_mbspbrk(path, "?*")) {
+	errno = ENOENT;
+	return -1;
+    }
+    h = FindFirstFile(path, &wfd);
+    if (h != INVALID_HANDLE_VALUE) {
+	FindClose(h);
+	st->st_mode  = fileattr_to_unixmode(wfd.dwFileAttributes, path);
+	st->st_atime = filetime_to_unixtime(&wfd.ftLastAccessTime);
+	st->st_mtime = filetime_to_unixtime(&wfd.ftLastWriteTime);
+	st->st_ctime = filetime_to_unixtime(&wfd.ftCreationTime);
+	st->st_size  = wfd.nFileSizeLow; /* TODO: 64bit support */
+    }
+    else {
+	// If runtime stat(2) is called for network shares, it fails on WinNT.
+	// Because GetDriveType returns 1 for network shares. (Win98 returns 4)
+	DWORD attr = GetFileAttributes(path);
+	if (attr == -1) {
+	    errno = map_errno(GetLastError());
+	    return -1;
+	}
+	st->st_mode  = fileattr_to_unixmode(attr, path);
+    }
+
+    st->st_dev = st->st_rdev = (isalpha(path[0]) && path[1] == ':') ?
+	toupper(path[0]) - 'A' : _getdrive() - 1;
+
+    return 0;
+}
+
 int
 rb_w32_stat(const char *path, struct stat *st)
 {
@@ -2859,7 +2949,7 @@ rb_w32_stat(const char *path, struct stat *st)
     } else if (*end == '\\' || (buf1 + 1 == end && *end == ':'))
 	strcat(buf1, ".");
 
-    ret = stat(buf1, st);
+    ret = IsWinNT() ? winnt_stat(buf1, st) : stat(buf1, st);
     if (ret == 0) {
 	st->st_mode &= ~(S_IWGRP | S_IWOTH);
     }
